---
layout: post
author: ryannystrom
title: Finding Your App Name
---

Coming up with app names is _hard_. I spent the last two weeks completely rebranding my app, [GitHawk](https://itunes.apple.com/us/app/githawk-for-github/id1252320249?mt=8). This post is a reflection on my process for generating ideas, iterating, and validating them.

## Backstory

I've been working on an open source GitHub app for a few months now. When I first released the app, I came up with what I thought was a clever name: "Freetime". Developers like to joke about how they have no free time to write open source, so this app was supposed to help get that time back.

But if you have even the slightest experience with App Store marketing, you probably _cringed_ seeing me use a name with the word "free" in it. Little did I realize that there were _thousands_ of other results.

That's why I recently [rebranded](http://blog.githawk.com/2017/08/16/Rebranding.html) to the name **GitHawk**, which I wont lie, is an _awesome_ name. In one word it:

- It expresses that the app does stuff with git
- A hawk is a fast, efficient, and beautiful animal that watches the world from above. Much like GitHawk helps you watch your GitHub projects!
- The capital "G" and "H" in **G**it**H**awk compliments **G**it**H**ub really well, especially when read as "GitHawk for GitHub"
- There are **zero** App Store search results when searching for "githawk"
- The [@githawk](https://twitter.com/githawk) Twitter handle was available, as well as [githawk.com](http://githawk.com) 😱

## What's in a name?

Creating the perfect app name feels impossible. There are names that work in your head but nobody else understands (_ahem_, "Freetime"). There are names which are _perfect_, only to discover they're taken for something totally unrelated.

After a while it feels pointless. Every good name is taken. Every other idea you have is awful.

Are there even any good app names left? Maybe I should just remove a bunch of vowels and slap a "ly" on the end.

_Don't give up!_ Finding the right app name is hard, but not impossible.

### Ground Rules

First, let's set some ground rules for picking a good name:

- There should be almost no results when searching the App Store
    + Results also shouldn't contain any competition!
- A Twitter handle should be available
    + Preferably **@name**, but **@nameapp** is ok too
- A **.com** domain should be available
    + You can be more loose with this
    + name.com, name-app.com, nameapp.com
    + Nobody goes to or remembers "name-app.io". Make it easy by sticking to .com

Whenever you have an idea, run it through this framework. If it doesn't pass all of these tests, just forget it and move on. The last thing you want is to get stuck with a great name that is saturated.

> This is my thought process. You might not care about Twitter, or maybe you need a Facebook page too. Adjust the rules depending on what you need.

I thought I struck gold when I came up with the name "Gitbox": there were no App Store results, and @gitboxapp was available on Twitter. I was so excited until I went to [gitbox.com](http://gitbox.com) and saw it was already a Mac app!

I could have grabbed `gitboxapp.com` and continued, but I predicted lots of confusion.

### Brainstorm

I like to write down _tons_ of possible app names that span a wide range of ideas.

**Start with competitors**

Research your competition and write down some keywords. For GitHawk, I searched "github" on the App Store and wrote down words that I saw in app titles. Things like "git", "code", "hub", "box", "fork", "branch", and "octo".

Not a terribly diverse set of words, but its a starting point.

I also studied the names themselves. _Does this name express what the app does?_ If the names were all over the place, I could probably get away with something more wild. 

However, most GitHub apps have pretty explicit names. That tells me I should avoid getting too weird with the name.

**[ProductHunt](http://wwwproducthunt.com) and [Dribbble](http://dribbble.com)**

I use these two sites **a lot** for idea generation. The content centers around apps, so simply by searching for a topic you can search through hundreds of other apps to discover new names.

Again I searched for "github" on both of these platforms. I found things like "chirp", "tree", "merge".

### Merge and Iterate

Go through the words that you wrote down and start prefixing and suffixing them with other words, either ones that you found or new ideas.

Take note of common patterns that start to emerge.

- Do you gravitate towards pairing a certain word (e.g. I was obsessed with appending "box" to everything)
- Are there words that compliment each other?
- What common themes start to emerge?

![Brainstorming names](https://user-images.githubusercontent.com/739696/29856302-c8e53aea-8d06-11e7-93d6-7d1f28e6707d.jpeg)

While you're doing this, some other words or ideas might pop up. **Write them down** but don't stop iterating! Those might be good ideas, but don't get distracted from mining your brain, there could be even more gems down there.

Keep doing this as more ideas come up. You should end up with dozens of options, with a couple of them standing out.

For me, there were a few pairs that I really liked:

- "Gitbox"
- "Gitmark"
- "Boltbox"

However, most of them were either taken or too weird.

### Start Branching

Scan over the names you like, and see if any other ideas pop up.

For me, I ran into things like "tree" and "branch". I started to think about nature. Git has "branches" and "trees", what else do trees do?

That's when I came up with the name "twig". I loved it! It's like a small tree, sort of like how my app is a small subset of GitHub.

However I couldn't get a good Twitter handle or domain, even though the App Store had very few results for "twig".

Continue to go back over all the brainstorming you've done. Sometimes new ideas will pop out.

### Take a breather

Now you might have some names that you're starting to gravitate towards. Maybe they sound cool. Or they're really expressive. Or maybe the name is totally unique, and the Twitter and domain are available.

Act fast, right?!?!

I actually grabbed gitmarkapp.com and @gitmark on Twitter and _almost_ released an update using this name. But after a night's rest I backed out of the idea.

You should obviously grab handles and domains if you really love the name, but don't immediately commit. Take a night to sleep on it, think about it in the shower. Go to work and think about it on breaks.

If you try to grind out a name overnight, you'll burn out and settle on a bad app name. Take a few days for a new name to sink in, and let other ideas blossom while you do other things.

### Finding GitHawk

After sitting on "gitmark" for a few days, I reread my notebook full of ideas when the word "sparrow" stood out in my notes next to some brainstorms about trees.

I explored this space: "bird", "tweet", "fly", "wing", "feather", "crow", "eagle", "hawk".

Wo, wait.

"Hawk".

Let's go back to my keywords and mash some together, does anything stick? I immediately ran into "GitHawk" and _loved_ it. It totally conveys what the app does.

Next up was Twitter: holy cow [@githawk](https://twitter.com/githawk) is available. Nabbed.

What about the domain? Somehow `githawk.com` is available and for only **$10/mo**?! Boom, done.

### End to End

All said and done, it took me almost a week to create a new name. I did **not** come up with "GitHawk" overnight, nor did the name just "come to me". I sludged through tons of _terrible_ names, mining for that one gem.

Having a way to seed ideas, branch new words, and setting ground rules gave me a framework to work towards a new app name. Without it, I could have gotten distracted, stressed out, or settled on a stupid name (_again_, "Freetime").

Hopefully this helps next time you're trying to name that new app!